################################################################################
#
# Copyright (c) <2012> <NETIKUS.NET ltd>
#
# Retrieves svn log comments and emails them to subscribers based on
# svn_history.ini configuration file. This script should be scheduled
# to run every HOURS_TO_QUERY hours.
#
# Configure this script in the CUSTOMIZE HERE section below.
#
# Version: 1.0
# Date:    2012-08-28
# Author:  NETIKUS.NET ltd (http://www.netikus.net)
# License: MIT License, see license.txt
#
################################################################################

use strict;

# Required Perl Modules
use Date::Manip;        # interpret date string from SVN
use Net::SMTP;          # send email through SMTP

############################# CUSTOMIZE HERE ###################################

# Path to your svn.exe executable, include full path if necessary
use constant SVN_EXE          => "svn.exe";

# The base of your SVN repository, all paths in ini files are appended to this path
use constant SVN_BASE         => "https://svn.yourdomain.local/svn";

# Your email suffix, e.g. \@abc.com
use constant EMAIL_SUFFIX     => "\@yourdomain.net";

# Your email server, all emails are sent through this host
use constant SMTP_SERVER      => "mail.yourdomain.local";

# The subject of all emails sent out, as well as sender name
use constant SMTP_SUBJECT     => "Daily Subversion History";
use constant SMTP_SENDER_NAME => "Subversion Admin";

# Match this number to the interval this script will be scheduled to run. In most
# cases you will want to run this early in the morning.
use constant HOURS_TO_QUERY   => 24;

############################# CUSTOMIZE HERE ###################################

use constant SMTP_SENDER      => "donotreply" . EMAIL_SUFFIX;
use constant SVNLINEDELIMITER => "------------------------------------------------------------------------";

################################################################################

my %svnFoldersHash;
my %emailContent;
my %uniqueUsers;

################################################################################

#
# Pass SVN username, returns email address. Customize this function if
# the SVN username is different from the email prefix
#
sub getEmailFromSvnUsername
{
    my ($svnUsername) = @_;
    
    my $emailAddress = $svnUsername . EMAIL_SUFFIX;
    
    return $emailAddress;
}

#
# Returns the command line syntax to display the svn log for a given folder
#
sub getCommandLineSyntax
{
    my ($folder) = @_;
    my $command = SVN_EXE . " log $folder -l 50 -v";
    return $command;
}

#
# Parse the .ini file, which maps SVN paths to recipients
#
sub parseIniFile
{
    open INIFILE, "<svn_history.ini" or die $!;
    
    my @ini_lines = <INIFILE>;
    my $currentUser = "";
    
    for (@ini_lines)
    {
        my $currentLine = $_;
        chomp($currentLine);
        
        if (substr($currentLine, 0, 1) eq "#")
            { next }
        
        if (length($currentLine) > 0)
        {
            if (!/\[.*\]/)
            {
                if (length($currentUser) > 0)
                {
                    $svnFoldersHash{$currentLine} .= $currentUser . ":";
                }
            }
            else
            {
                ($currentUser) = ($currentLine =~ /\[(.*)\]/);
            }
        }
    }
    
    close INIFILE;
}

#
# Execute svn, and parse the "svn log" output
#
sub parseSvnLogOutput
{
    my $dateFrom = time() - (HOURS_TO_QUERY * 3600);

    foreach my $path (keys %svnFoldersHash)
    {
        my $svnPath = SVN_BASE . "/" . $path;
        my $cmd = getCommandLineSyntax($svnPath);
        
        my $result = `$cmd`;
        my @lines = split("\n", $result);
        
        my @users = split(/:/, $svnFoldersHash{$path});
        for (@users)
        {
            my $newEntry = 0;
            my $recordComments = 0;
            
            my $currentUser = $_;
            $uniqueUsers{$currentUser} = 1;
            
            my $commentPrinted = 0;
            my $commentCount = 0;
            
            for (@lines)
            {
                if ($recordComments == 1 && 
                    $_ ne SVNLINEDELIMITER)
                {
                    if (length($_) > 0) 
                    {
                        if ($commentPrinted == 1) 
                        {
                            $commentCount++;
                            
                            if ($commentCount >= 2) {
                                $emailContent{$currentUser} .= "   ";
                            }
                        }
                        
                        $emailContent{$currentUser} .= "$_\n";
                    }
                    else {
                        if ($commentPrinted == 0)
                        {
                            $emailContent{$currentUser} .= "\nComments:\n   ";
                            $commentPrinted = 1;
                        }
                    }
                }
    
                if ($newEntry == 1)
                {
                    my @properties = split(/\|/, $_);
                    
                    my ($date, $time) = split(" ", $properties[2]);
                    my $user = $properties[1];
                    
                    my $datetime = "$date $time";
                    my $unixDate = UnixDate($datetime, "%s");
                    
                    if ($unixDate >= $dateFrom)
                    {
                        $user =~ s/ //g;
                        
                        if ($currentUser ne $user)
                        {
                            $emailContent{$currentUser} .= "\n=============================================================\n";
                            $emailContent{$currentUser} .= "$date $time\n$user committed $properties[0]\n\n";
                            
                            $recordComments = 1;
                        }
                    }
                    else {
                        last;
                    }
                }
                
                if ($_ eq SVNLINEDELIMITER)
                {
                    $commentPrinted = 0;
                    $recordComments = 0;
                    $commentCount   = 0;
                    $newEntry = 1;
                }
                else
                {
                    $newEntry = 0;
                }
            }
        }
    }
}

#
# Sends out emails to all subscribers
#
sub sendEmails()
{
    foreach my $user (keys %uniqueUsers)
    {
        if (length($emailContent{$user}) > 0)
        {
            my $smtp = Net::SMTP->new(SMTP_SERVER);
            
            $smtp->mail(SMTP_SENDER);
            $smtp->recipient(getEmailFromSvnUsername($user));
            
            $smtp->data();
            $smtp->datasend("From: " . SMTP_SENDER_NAME . " <" . SMTP_SENDER . ">\n");
            $smtp->datasend("To: $user <$user" . EMAIL_SUFFIX . ">\n");
            $smtp->datasend("Subject: " . SMTP_SUBJECT . "\n");
            
            $smtp->datasend($emailContent{$user});
            
            $smtp->dataend;
            $smtp->quit;
        }
    }
}

################################################################################
################################################################################

Date_Init("TZ=CDT");

parseIniFile();

parseSvnLogOutput();

sendEmails();
